#include "glwindow.h"
#include <math.h>

CGLWindow window;

void DrawCube()
{
	// Colorful Quad
	glBegin(GL_QUAD_STRIP);
	glColor3f(1,0,0);
	glVertex3f(-0.5f,-0.5f,-0.5f);
	glVertex3f( 0.5f,-0.5f,-0.5f);
	glColor3f(1,1,0);
	glVertex3f(-0.5f, 0.5f,-0.5f);
	glVertex3f( 0.5f, 0.5f,-0.5f);
	glColor3f(0,1,0);
	glVertex3f(-0.5f, 0.5f, 0.5f);
	glVertex3f( 0.5f, 0.5f, 0.5f);
	glColor3f(0,1,1);
	glVertex3f(-0.5f,-0.5f, 0.5f);
	glVertex3f( 0.5f,-0.5f, 0.5f);
	glColor3f(1,0,1);
	glVertex3f(-0.5f,-0.5f,-0.5f);
	glVertex3f( 0.5f,-0.5f,-0.5f);
	glEnd();

	glColor3f(0,0,1);
	glBegin(GL_QUADS);
	glVertex3f(-0.5f,-0.5f,-0.5f);
	glVertex3f(-0.5f, 0.5f,-0.5f);
	glVertex3f(-0.5f, 0.5f, 0.5f);
	glVertex3f(-0.5f,-0.5f, 0.5f);

	glVertex3f( 0.5f,-0.5f,-0.5f);
	glVertex3f( 0.5f, 0.5f,-0.5f);
	glVertex3f( 0.5f, 0.5f, 0.5f);
	glVertex3f( 0.5f,-0.5f, 0.5f);
	glEnd();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90.0f, 1.3f, 0.1f, 1000.0f);
	glMatrixMode(GL_MODELVIEW);
}

void Initialize()
{
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_FLAT);
}

void Render()
{
	float alpha = GetTickCount()/10.0f;
	float scale = sinf(GetTickCount()/1000.0f)+1.1f;

	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	glTranslatef(0,0,-4);

	glPushMatrix();
	glTranslatef(-2,-2,0);
	glRotatef(alpha,1.0f,0.5f,0.0f);
	DrawCube();
	glPopMatrix();

	glPushMatrix();
	glTranslatef(2,-2,0);
	glRotatef(alpha,1.0f,-0.5f,0.5f);
	DrawCube();
	glPopMatrix();

	glPushMatrix();
	glTranslatef(2, 2,0);
	glRotatef(alpha,0.0f,0.5f,-1.0f);
	DrawCube();
	glPopMatrix();


	glPushMatrix();
	glTranslatef(-2, 2,0);
	glRotatef(alpha,1.0f,-0.5f,1.0f);
	DrawCube();
	glPopMatrix();
	
	glPushMatrix();
	glScalef(scale,scale,scale);
	glRotatef(alpha,-0.5f,1.0f,0.0f);
	DrawCube();
	glPopMatrix();
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_KEYDOWN:
			switch(wParam)
			{
				case VK_ESCAPE:
					PostQuitMessage(0);
					break;
				default:
					break;
			}
			break;
		case WM_DESTROY:
		case WM_CLOSE:
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			window.Resize(LOWORD(lParam), HIWORD(lParam));
			break;
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int iCmdShow)
{
	MSG msg;
	bool done = false;

	window.Initialize(800, 600, 32, false);

	Initialize();

	while(!done){
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)){
			if(msg.message==WM_QUIT){
				done = true;
			}
			else{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
		else{
			Render();
			window.Swap();
		}
	}

	window.Terminate();
	
	return msg.wParam;
}
